use std::io::Read;

const BYTE_FACTOR: usize = 2;

/// Read every other byte from `R`.
///
/// If `R` is a stream of u16be, this should return the least-significant byte of each u16.
pub struct StreamLSB<R: Read> {
    /// The source to read from
    src: R,

    /// Number of filled bits in fragbuf
    arity: usize,

    /// Hold a partial u16, in case we read half of one
    fragbuf: [u8; BYTE_FACTOR],
}

impl<R: Read> StreamLSB<R> {
    pub fn new(src: R) -> Self {
        Self {
            src,
            arity: 0,
            fragbuf: [0; BYTE_FACTOR],
        }
    }
}

impl<R: Read> Read for StreamLSB<R> {
    fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
        let n_raw = self.src.read(buf)?;

        let mut in_cursor = 0;
        let mut out_cursor = 0;

        if self.arity != 0 {
            // Number of bytes we need to read
            let need_read = BYTE_FACTOR - self.arity;
            let can_read = need_read.min(buf.len());
            self.fragbuf[self.arity..self.arity + can_read].copy_from_slice(&buf[0..can_read]);

            in_cursor += can_read;
            self.arity += can_read;
            self.arity %= BYTE_FACTOR;

            if self.arity != 0 {
                // We did not read a full set of bytes,
                // wait for more data.
                return Ok(0);
            } else {
                // Extract least-significant byte
                buf[0] = self.fragbuf[0];
                out_cursor += 1;
            }
        }

        for _ in in_cursor..(n_raw / BYTE_FACTOR) {
            // Extract least-significant byte
            buf[out_cursor] = buf[in_cursor];

            in_cursor += BYTE_FACTOR;
            out_cursor += 1;
        }

        self.arity = n_raw - in_cursor;
        self.fragbuf[0..self.arity].copy_from_slice(&buf[in_cursor..n_raw]);

        return Ok(out_cursor);
    }
}
